pub enum RainDensity {
Absolute {
num_drops: usize,
},
Relative {
sparseness: usize,
},
Dense,
Normal,
Sparse,
}Expand description
A configuration for the density of the rain effect.
Variants
Absolute
An absolute target number of drops to have in the frame.
Relative
Compute the number of drops based on the frame size. Lower value is denser.
Is converted to an absolute value, with 1 drop per sparseness pixels.
Dense
A dense rain. Equivalent to Relative { sparseness: 20 }.
Normal
A normal rain. Equivalent to Relative { sparseness: 50 }.
Sparse
A sparse rain. Equivalent to Relative { sparseness: 100 }.